import Flutter
import UIKit
import ZHNScan

public class SwiftZhnScanPlugin: NSObject, FlutterPlugin {
    
    private var result: FlutterResult?
    private let vc = ScanVC()
    private var hostViewController: UIViewController?
    
    public static func register(with registrar: FlutterPluginRegistrar) {
        let channel = FlutterMethodChannel(name: "zhn_scan", binaryMessenger: registrar.messenger())
        let instance = SwiftZhnScanPlugin()
        if let window = UIApplication.shared.delegate?.window, let rootViewController = window?.rootViewController{
            instance.hostViewController = rootViewController
        } else {
            instance.hostViewController = UIViewController()
        }
        instance.hostViewController = UIApplication.shared.delegate?.window!?.rootViewController
        registrar.addMethodCallDelegate(instance, channel: channel)
    }

    public func handle(_ call: FlutterMethodCall, result: @escaping FlutterResult) {
        if call.method == "scan" {
            self.result = result
            self.showScanView()
        } else if call.method == "getPlatformVersion"  {
            result("iOS " + UIDevice.current.systemVersion)
        } else {
            result(FlutterMethodNotImplemented)
        }
    }
    
    func showScanView() {
        let kNavBarColor                 =        #colorLiteral(red: 0.1843137255, green: 0.1882352941, blue: 0.1960784314, alpha: 1)
        let navigationController = UINavigationController(rootViewController: vc)
        navigationController.navigationBar.isTranslucent = true
        navigationController.navigationBar.barStyle = .black
        navigationController.navigationBar.barTintColor = kNavBarColor
        navigationController.navigationBar.titleTextAttributes = [ .foregroundColor : UIColor.white ]
        
        vc.title = "扫描二维码"
        
        vc.scanClosure = { [weak self] codeString in
            guard let `self` = self else { return }
            guard let result = self.result else {
                return
            }
            result(codeString)
        }
        
        createNavButton(imageName: "backIcon", action: #selector(leftNavButtonAction), vc: vc)
        
        self.hostViewController?.present(navigationController, animated: false, completion: nil)
        
    }
    
    /// 创建导航栏图片按钮
    private func createNavButton(imageName name: String, isLeft: Bool = true, action: Selector, vc: UIViewController) {
        let image = UIImage()
        let item = UIBarButtonItem(image: image, style: .plain, target: self, action: action)
        if isLeft {
            vc.navigationItem.leftBarButtonItem = item
        } else {
            vc.navigationItem.rightBarButtonItem = item
        }
    }
    
    @objc private func leftNavButtonAction() {
        self.hostViewController?.dismiss(animated: true, completion: nil)
    }
}
